@@ -1,3 +1,4 @@
+import os
 import argparse
 import sys
 import signal
@@ -314,6 +315,115 @@ class Command(Command_co):
         self._out.writeln('Sense2 commands. Type "sense2 <command> -h" for help', Color.BRIGHT_YELLOW)   
     
 
+    def upload(self, *args):
+        """
+        Uploads a recipe and binary packages to a remote.
+
+        If no remote is specified, the first configured remote (by default conan-center, use
+        'conan remote list' to list the remotes) will be used.
+        """
+        parser = argparse.ArgumentParser(description=self.upload.__doc__,
+                                         prog="sense2 upload",
+                                         formatter_class=SmartFormatter)
+        parser.add_argument('pattern_or_reference', help=_PATTERN_REF_OR_PREF_HELP)
+        parser.add_argument("-p", "--package", default=None,
+                            help="Package ID [DEPRECATED: use full reference instead]",
+                            action=OnceArgument)
+        parser.add_argument('-q', '--query', default=None, action=OnceArgument,
+                            help="Only upload packages matching a specific query. " + _QUERY_HELP)
+        parser.add_argument("-r", "--remote", action=OnceArgument,
+                            help='upload to this specific remote')
+        parser.add_argument("--all", action='store_true', default=True,
+                            help='Upload both package recipe and packages')
+        parser.add_argument("--skip-upload", action='store_true', default=False,
+                            help='Do not upload anything, just run the checks and the compression')
+        parser.add_argument("--force", action='store_true', default=False,
+                            help='Ignore checks before uploading the recipe: it will bypass missing'
+                                 ' fields in the scm attribute and it will override remote recipe'
+                                 ' with local regardless of recipe date')
+        parser.add_argument("--check", action='store_true', default=False,
+                            help='Perform an integrity check, using the manifests, before upload')
+        parser.add_argument('-c', '--confirm', default=False, action='store_true',
+                            help='Upload all matching recipes without confirmation')
+        parser.add_argument('--retry', default=None, type=int, action=OnceArgument,
+                            help="In case of fail retries to upload again the specified times.")
+        parser.add_argument('--retry-wait', default=None, type=int, action=OnceArgument,
+                            help='Waits specified seconds before retry again')
+        parser.add_argument("-no", "--no-overwrite", nargs="?", type=str, choices=["all", "recipe"],
+                            action=OnceArgument, const="all",
+                            help="Uploads package only if recipe is the same as the remote one")
+        parser.add_argument("-j", "--json", default=None, action=OnceArgument,
+                            help='json file path where the upload information will be written to')
+        parser.add_argument("--parallel", action='store_true', default=False,
+                            help='Upload files in parallel using multiple threads. '
+                                 'The default number of launched threads is set to the value of '
+                                 'cpu_count and can be configured using the CONAN_CPU_COUNT '
+                                 'environment variable or defining cpu_count in conan.conf')
+
+        args = parser.parse_args(*args)
+
+        try:
+            pref = PackageReference.loads(args.pattern_or_reference, validate=True)
+        except ConanException:
+            reference = args.pattern_or_reference
+            package_id = args.package
+
+            if package_id:
+                self._out.warn("Usage of `--package` argument is deprecated."
+                               " Use a full reference instead: "
+                               "`conan upload [...] {}:{}`".format(reference, package_id))
+
+            if args.query and package_id:
+                raise ConanException("'--query' argument cannot be used together with '--package'")
+        else:
+            reference = repr(pref.ref)
+            package_id = "{}#{}".format(pref.id, pref.revision) if pref.revision else pref.id
+
+            if args.package:
+                raise ConanException("Use a full package reference (preferred) or the `--package`"
+                                     " command argument, but not both.")
+            if args.query:
+                raise ConanException("'--query' argument cannot be used together with "
+                                     "full reference")
+
+        if args.force and args.no_overwrite:
+            raise ConanException("'--no-overwrite' argument cannot be used together with '--force'")
+        if args.force and args.skip_upload:
+            raise ConanException("'--skip-upload' argument cannot be used together with '--force'")
+        if args.no_overwrite and args.skip_upload:
+            raise ConanException("'--skip-upload' argument cannot be used together "
+                                 "with '--no-overwrite'")
+
+        self._warn_python_version()
+
+        if args.force:
+            policy = UPLOAD_POLICY_FORCE
+        elif args.no_overwrite == "all":
+            policy = UPLOAD_POLICY_NO_OVERWRITE
+        elif args.no_overwrite == "recipe":
+            policy = UPLOAD_POLICY_NO_OVERWRITE_RECIPE
+        elif args.skip_upload:
+            policy = UPLOAD_POLICY_SKIP
+        else:
+            policy = None
+
+        info = None
+        try:
+            info = self._conan.upload(pattern=reference, package=package_id,
+                                      query=args.query, remote_name=args.remote,
+                                      all_packages=args.all, policy=policy,
+                                      confirm=args.confirm, retry=args.retry,
+                                      retry_wait=args.retry_wait, integrity_check=args.check,
+                                      parallel_upload=args.parallel)
+
+        except ConanException as exc:
+            info = exc.info
+            raise
+        finally:
+            if args.json and info:
+                self._outputer.json_output(info, args.json, os.getcwd())
+
+
 def main(args):
     """ main entry point of the conan application, using a Command to
     parse parameters
