@@ -9,6 +9,7 @@ from conans.errors import (ConanException, RecipeNotFoundException,
 from sense2.client.conanfile.deb import run_deb_method
 from sense2.client.cache.cache import ClientCache
 from conans.client.recorder.upload_recoder import UploadRecorder
+from sense2.client.cmd.uploader import CmdUpload
 from sense2.client.cmd.uploader_deb import CmdUploadDeb
 from conans.client.recorder.action_recorder import ActionRecorder
 from conans.client.cmd.export import cmd_export
@@ -113,6 +114,25 @@ class Sense2APIV1(ConanAPIV1_co):
             exc.info = recorder.get_info(self.app.config.revisions_enabled)
             raise
     
+    @api_method
+    def upload(self, pattern, package=None, remote_name=None, all_packages=False, confirm=False,
+               retry=None, retry_wait=None, integrity_check=False, policy=None, query=None,
+               parallel_upload=False):
+        """ Uploads a package recipe and the generated binary packages to a specified remote
+        """
+        upload_recorder = UploadRecorder()
+        uploader = CmdUpload(self.app.cache, self.app.user_io, self.app.remote_manager,
+                             self.app.loader, self.app.hook_manager)
+        remotes = self.app.load_remotes(remote_name=remote_name)
+        try:
+            uploader.upload(pattern, remotes, upload_recorder, package, all_packages, confirm,
+                            retry, retry_wait, integrity_check, policy, query=query,
+                            parallel_upload=parallel_upload)
+            return upload_recorder.get_info()
+        except ConanException as exc:
+            upload_recorder.error = True
+            exc.info = upload_recorder.get_info()
+            raise
     
     @api_method
     def install_deb(self, conanfile_path, info_folder=None, cwd=None):
